/*
 * Copyright (C) 2005-2006 mark307
 * See Copyright Notice in COPYING.LUA
 */

#include <windows.h>
#include <windowsx.h>
#include <commdlg.h>
#include <shlobj.h>
#include <string.h>

#include <lua.h>
#include <lauxlib.h>

#include "select_directory.h"

#define LIBNAME "w32cd"


/**
   
   * Win32 Common Dialog functions
   
   ** Defined variables
   
   OFN_ALLOWMULTISELECT OFN_CREATEPROMPT OFN_ENABLESIZING OFN_EXPLORER
   OFN_EXTENSIONDIFFERENT OFN_FILEMUSTEXIST OFN_HIDEREADONLY OFN_LONGNAMES
   OFN_NOCHANGEDIR OFN_NODEREFERENCELINKS OFN_NOLONGNAMES OFN_NONETWORKBUTTON
   OFN_NOREADONLYRETURN OFN_NOTESTFILECREATE OFN_NOVALIDATE OFN_PATHMUSTEXIST
   OFN_READONLY OFN_SHAREAWARE OFN_SHOWHELP OFN_SHAREFALLTHROUGH
   OFN_SHARENOWARN OFN_SHAREWARN
   
      GetOpenFileName, GetSaveFileName flags
   
   CC_RGBINIT CC_FULLOPEN CC_PREVENTFULLOPEN CC_SHOWHELP CC_SOLIDCOLOR
   CC_ANYCOLOR
   
      ChooseColor flags
   
   CF_SCREENFONTS CF_PRINTERFONTS CF_BOTH CF_SHOWHELP CF_INITTOLOGFONTSTRUCT
   CF_EFFECTS CF_APPLY CF_ANSIONLY CF_SCRIPTSONLY CF_NOVECTORFONTS
   CF_NOOEMFONTS CF_NOSIMULATIONS CF_LIMITSIZE CF_FIXEDPITCHONLY CF_WYSIWYG
   CF_FORCEFONTEXIST CF_SCALABLEONLY CF_TTONLY CF_NOFACESEL CF_NOSTYLESEL
   CF_NOSIZESEL CF_SELECTSCRIPT CF_NOSCRIPTSEL CF_NOVERTFONTS
   
      ChooseFont flags
   
   
   ** Defined functions
   
*/

static struct {
	const char *name;
	DWORD value;
} w32cd_consts[] = {
	/*
	 * Common Dialog
	 */
	/* OPENFILENAME */
	{"OFN_ALLOWMULTISELECT",   OFN_ALLOWMULTISELECT},
	{"OFN_CREATEPROMPT",       OFN_CREATEPROMPT},
	{"OFN_ENABLESIZING",       OFN_ENABLESIZING},
	{"OFN_EXPLORER",           OFN_EXPLORER},
	{"OFN_EXTENSIONDIFFERENT", OFN_EXTENSIONDIFFERENT},
	{"OFN_FILEMUSTEXIST",      OFN_FILEMUSTEXIST},
	{"OFN_HIDEREADONLY",       OFN_HIDEREADONLY},
	{"OFN_LONGNAMES",          OFN_LONGNAMES},
	{"OFN_NOCHANGEDIR",        OFN_NOCHANGEDIR},
	{"OFN_NODEREFERENCELINKS", OFN_NODEREFERENCELINKS},
	{"OFN_NOLONGNAMES",        OFN_NOLONGNAMES},
	{"OFN_NONETWORKBUTTON",    OFN_NONETWORKBUTTON},
	{"OFN_NOREADONLYRETURN",   OFN_NOREADONLYRETURN},
	{"OFN_NOTESTFILECREATE",   OFN_NOTESTFILECREATE},
	{"OFN_NOVALIDATE",         OFN_NOVALIDATE},
//	{"OFN_OVERWRITEPROMPT",    OFN_OVERWRITEPROPT},
	{"OFN_PATHMUSTEXIST",      OFN_PATHMUSTEXIST},
	{"OFN_READONLY",           OFN_READONLY},
	{"OFN_SHAREAWARE",         OFN_SHAREAWARE},
	{"OFN_SHOWHELP",           OFN_SHOWHELP},
	{"OFN_SHAREFALLTHROUGH",   OFN_SHAREFALLTHROUGH},
	{"OFN_SHARENOWARN",        OFN_SHARENOWARN},
	{"OFN_SHAREWARN",          OFN_SHAREWARN},
	/* CHOOSECOLOR */
	{"CC_RGBINIT", CC_RGBINIT},
	{"CC_FULLOPEN", CC_FULLOPEN},
	{"CC_PREVENTFULLOPEN", CC_PREVENTFULLOPEN},
	{"CC_SHOWHELP", CC_SHOWHELP},
	{"CC_SOLIDCOLOR", CC_SOLIDCOLOR},
	{"CC_ANYCOLOR", CC_ANYCOLOR},
	/* CHOOSEFONT */
	{"CF_SCREENFONTS", CF_SCREENFONTS},
	{"CF_PRINTERFONTS", CF_PRINTERFONTS},
	{"CF_BOTH", CF_BOTH},
	{"CF_SHOWHELP", CF_SHOWHELP},
	{"CF_INITTOLOGFONTSTRUCT", CF_INITTOLOGFONTSTRUCT},
//	{"CF_USESTYLE", CF_USESTYLE},
	{"CF_EFFECTS", CF_EFFECTS},
	{"CF_APPLY", CF_APPLY},
	{"CF_ANSIONLY", CF_ANSIONLY},
	{"CF_SCRIPTSONLY", CF_SCRIPTSONLY},
	{"CF_NOVECTORFONTS", CF_NOVECTORFONTS},
	{"CF_NOOEMFONTS", CF_NOOEMFONTS},
	{"CF_NOSIMULATIONS", CF_NOSIMULATIONS},
	{"CF_LIMITSIZE", CF_LIMITSIZE},
	{"CF_FIXEDPITCHONLY", CF_FIXEDPITCHONLY},
	{"CF_WYSIWYG", CF_WYSIWYG},
	{"CF_FORCEFONTEXIST", CF_FORCEFONTEXIST},
	{"CF_SCALABLEONLY", CF_SCALABLEONLY},
	{"CF_TTONLY", CF_TTONLY},
	{"CF_NOFACESEL", CF_NOFACESEL},
	{"CF_NOSTYLESEL", CF_NOSTYLESEL},
	{"CF_NOSIZESEL", CF_NOSIZESEL},
	{"CF_SELECTSCRIPT", CF_SELECTSCRIPT},
	{"CF_NOSCRIPTSEL", CF_NOSCRIPTSEL},
	{"CF_NOVERTFONTS", CF_NOVERTFONTS},
	/* end */
	{NULL, 0},
};


/**
   ChooseColor
   
   Wrapper of ChooseColor Win32 Common Dialog API
   
      color, cc = ChooseColor(hwnd [, cc])
   
   hwnd  : integer : Window Handle of owner windows
   cc    : table   : CHOOSECOLOR structure
   color : integer : color.
   
   If user selects cancel, this function will return nil.
   
   members of cc
   RGB        : integer : color
   Flags      : integer : 
   CustColors : table   : array of custom colors
*/
static int L_ChooseColor(lua_State *L)
{
	CHOOSECOLOR cc = {0};
	DWORD lpCustColors[16] = {0};
	int i;
	
	lua_settop(L, 2);
	cc.hwndOwner = (HWND) luaL_checklong(L, 1);
	cc.lStructSize = sizeof(cc);
	cc.lpCustColors = lpCustColors;
	
	if (lua_istable(L, 2)) {
		lua_getfield(L, 2, "RGB");
		cc.rgbResult = luaL_optlong(L, -1, 0);
		lua_pop(L, 1);
		
		lua_getfield(L, 2, "Flags");
		cc.Flags = luaL_optlong(L, -1, 0);
		lua_pop(L, 1);
		
		lua_getfield(L, 2, "CustColors");
		if (lua_istable(L, -1)) {
			for (i = 0; i < 16; i ++) {
				lua_pushinteger(L, i + 1);
				lua_gettable(L, -2);
				lpCustColors[i] = luaL_optlong(L, -1, 0);
				lua_pop(L, 1);
			}
		}
		lua_pop(L, 1);
	}
	lua_settop(L, 0);
	
	cc.Flags &= ~(CC_ENABLEHOOK | CC_ENABLETEMPLATE |
	              CC_ENABLETEMPLATEHANDLE);
	
	if (ChooseColor(&cc)) {
		lua_pushinteger(L, cc.rgbResult);
	} else {
		lua_pushnil(L);
	}
	
	lua_newtable(L);
	
	lua_pushinteger(L, cc.rgbResult);
	lua_setfield(L, -2, "RGB");
	
	lua_pushinteger(L, cc.Flags);
	lua_setfield(L, -2, "Flags");
	
	lua_newtable(L);
	for (i = 0; i < 16; i ++) {
		lua_pushinteger(L, i + 1);
		lua_pushinteger(L, lpCustColors[i]);
		lua_settable(L, -3);
	}
	lua_setfield(L, -2, "CustColors");
	
	return 2;
}


/**
   ChooseFont
   
   Wrapper of ChooseFont Win32 Common Dialog API
   
      ret, cf = ChooseFont(hwnd [, cf])
   
   hwnd  : integer : Window Handle of owner windows
   cf    : table   : CHOOSEFFONT structure
   ret   : boolean : If user selects cancel, ret will be false,
                     or else will be true.
   
   members of cf
   DC        : integer : HDC
   LogFont   : table   : LOGFONT structure
   PointSize : integer : 
   Flags     : integer : 
   Colors    : integer : 
   FontType  : integer : 
   SizeMin   : integer : 
   SizeMax   : integer : 
   
   members of LogFont
   Height         : integer : 
   Width          : integer : 
   Escapement     : integer : 
   Orientation    : integer : 
   Weight         : integer : 
   Italic         : integer : 
   Underline      : integer : 
   StrikeOut      : integer : 
   CharSet        : integer : 
   OutPrecision   : integer : 
   OutPrecision   : integer : 
   ClipPrecision  : integer : 
   Quality        : integer : 
   PitchAndFamily : integer : 
   FaceName       : string  : 
*/

static void table2logfont(lua_State *, int, LPLOGFONT);
static void logfont2table(lua_State *, int, LPLOGFONT);

int L_ChooseFont(lua_State *L)
{
	CHOOSEFONT cf = {0};
	LOGFONT lf = {0};
	
	cf.hwndOwner = (HWND) luaL_checklong(L, 1);
	cf.lStructSize = sizeof(cf);
	cf.lpLogFont = &lf;
	
	if (lua_istable(L, 2)) {
		lua_getfield(L, 2, "DC");
		cf.hDC = (HDC) luaL_optlong(L, -1, 0);
		lua_pop(L, 1);
		
		lua_getfield(L, 2, "LogFont");
		if (lua_istable(L, -1)) {
			table2logfont(L, -1, &lf);
		}
		lua_pop(L, 1);
		
		lua_getfield(L, 2, "PointSize");
		cf.iPointSize = luaL_optlong(L, -1, 0);
		lua_pop(L, 1);
		
		lua_getfield(L, 2, "Flags");
		cf.Flags = luaL_optlong(L, -1, 0);
		lua_pop(L, 1);
		
		lua_getfield(L, 2, "Colors");
		cf.rgbColors = luaL_optlong(L, -1, 0);
		lua_pop(L, 1);
		
		lua_getfield(L, 2, "SizeMin");
		cf.nSizeMin = luaL_optlong(L, -1, 0);
		lua_pop(L, 1);
		
		lua_getfield(L, 2, "SizeMax");
		cf.nSizeMax = luaL_optlong(L, -1, 0);
		lua_pop(L, 1);
	}
	lua_settop(L, 0);
	
	cf.Flags &= ~(CF_ENABLEHOOK | CF_ENABLETEMPLATE |
	              CF_ENABLETEMPLATEHANDLE | CF_USESTYLE);
	
	lua_pushboolean(L, ChooseFont(&cf));
	
	lua_newtable(L);
	
	lua_pushinteger(L, (lua_Integer) cf.hDC);
	lua_setfield(L, -2, "DC");
	
	lua_newtable(L);
	logfont2table(L, -2, &lf);
	lua_setfield(L, -2, "LogFont");
	
	lua_pushinteger(L, cf.iPointSize);
	lua_setfield(L, -2, "PointSize");
	
	lua_pushinteger(L, cf.Flags);
	lua_setfield(L, -2, "Flags");
	
	lua_pushinteger(L, cf.rgbColors);
	lua_setfield(L, -2, "Colors");
	
	lua_pushinteger(L, cf.nFontType);
	lua_setfield(L, -2, "FontType");
	
	lua_pushinteger(L, cf.nSizeMin);
	lua_setfield(L, -2, "SizeMin");
	
	lua_pushinteger(L, cf.nSizeMax);
	lua_setfield(L, -2, "SizeMax");
	
	return 2;
}


static void table2logfont(lua_State *L, int index, LPLOGFONT lplf)
{
	lua_getfield(L, index, "Height");
	lplf->lfHeight = luaL_optlong(L, -1, 0);
	lua_pop(L, 1);
	
	lua_getfield(L, index, "Width");
	lplf->lfWidth = luaL_optlong(L, -1, 0);
	lua_pop(L, 1);
	
	lua_getfield(L, index, "Escapement");
	lplf->lfEscapement = luaL_optlong(L, -1, 0);
	lua_pop(L, 1);
	
	lua_getfield(L, index, "Orientation");
	lplf->lfOrientation = luaL_optlong(L, -1, 0);
	lua_pop(L, 1);
	
	lua_getfield(L, index, "Weight");
	lplf->lfWeight = luaL_optlong(L, -1, 0);
	lua_pop(L, 1);
	
	lua_getfield(L, index, "Italic");
	lplf->lfItalic = (BYTE) luaL_optlong(L, -1, 0);
	lua_pop(L, 1);
	
	lua_getfield(L, index, "Underline");
	lplf->lfUnderline = (BYTE) luaL_optlong(L, -1, 0);
	lua_pop(L, 1);
	
	lua_getfield(L, index, "StrikeOut");
	lplf->lfStrikeOut = (BYTE) luaL_optlong(L, -1, 0);
	lua_pop(L, 1);
	
	lua_getfield(L, index, "CharSet");
	lplf->lfCharSet = (BYTE) luaL_optlong(L, -1, 0);
	lua_pop(L, 1);
	
	lua_getfield(L, index, "OutPrecision");
	lplf->lfOutPrecision = (BYTE) luaL_optlong(L, -1, 0);
	lua_pop(L, 1);
	
	lua_getfield(L, index, "OutPrecision");
	lplf->lfOutPrecision = (BYTE) luaL_optlong(L, -1, 0);
	lua_pop(L, 1);
	
	lua_getfield(L, index, "ClipPrecision");
	lplf->lfClipPrecision = (BYTE) luaL_optlong(L, -1, 0);
	lua_pop(L, 1);
	
	lua_getfield(L, index, "Quality");
	lplf->lfQuality = (BYTE) luaL_optlong(L, -1, 0);
	lua_pop(L, 1);
	
	lua_getfield(L, index, "PitchAndFamily");
	lplf->lfPitchAndFamily = (BYTE) luaL_optlong(L, -1, 0);
	lua_pop(L, 1);
	
	lua_getfield(L, index, "FaceName");
	strncpy(lplf->lfFaceName, luaL_optstring(L, -1, ""), LF_FACESIZE - 1);
	lua_pop(L, 1);
}


static void logfont2table(lua_State *L, int index, LPLOGFONT lplf)
{
	lua_pushinteger(L, lplf->lfHeight);
	lua_setfield(L, index, "Height");
	
	lua_pushinteger(L, lplf->lfWidth);
	lua_setfield(L, index, "Width");
	
	lua_pushinteger(L, lplf->lfEscapement);
	lua_setfield(L, index, "Escapement");
	
	lua_pushinteger(L, lplf->lfOrientation);
	lua_setfield(L, index, "Orientation");
	
	lua_pushinteger(L, lplf->lfWeight);
	lua_setfield(L, index, "Weight");
	
	lua_pushinteger(L, lplf->lfItalic);
	lua_setfield(L, index, "Italic");
	
	lua_pushinteger(L, lplf->lfUnderline);
	lua_setfield(L, index, "Underline");
	
	lua_pushinteger(L, lplf->lfStrikeOut);
	lua_setfield(L, index, "StrikeOut");
	
	lua_pushinteger(L, lplf->lfCharSet);
	lua_setfield(L, index, "CharSet");
	
	lua_pushinteger(L, lplf->lfOutPrecision);
	lua_setfield(L, index, "OutPrecision");
	
	lua_pushinteger(L, lplf->lfOutPrecision);
	lua_setfield(L, index, "OutPrecision");
	
	lua_pushinteger(L, lplf->lfClipPrecision);
	lua_setfield(L, index, "ClipPrecision");
	
	lua_pushinteger(L, lplf->lfQuality);
	lua_setfield(L, index, "Quality");
	
	lua_pushinteger(L, lplf->lfPitchAndFamily);
	lua_setfield(L, index, "PitchAndFamily");
	
	lua_pushstring(L, lplf->lfFaceName);
	lua_setfield(L, index, "FaceName");
}


/**
   GetOpenFileName, GetSaveFileName
   
   Wrapper of GetOpenFileName, GetSaveFileName Win32 Common Dialog API
   
      name, ofn = GetOpenFileName(hwnd [, ofn])
      name, ofn = GetSaveFileName(hwnd [, ofn])
   
   hwnd  : integer : Window Handle of owner windows
   ofn   : table   : OPENFILENAME structure
   name  : string  : filename selecred by user
   
   If user selects cancel, name will be nil.
   
   members of ofn
   Filter         : string  : 
   CustomFilter   : string  : 
   FilterIndex    : integer : 
   File           : string  : 
   InitialDir     : string  : 
   Title          : string  : 
   Flags          : integer : 
   FileOffset     : integer : 
   nFileExtension : integer : 
   DefExt         : string  : 
*/
static int openfilename(lua_State *L, BOOL WINAPI (*GetFileName)(LPOPENFILENAME))
{
	OPENFILENAME ofn = {0};
	char lpFile[MAX_PATH] = {0};
	char lpCustomFilter[42] = {0};
	
	lua_settop(L, 2);
	ofn.hwndOwner = (HWND) luaL_checklong(L, 1);
#if defined(WINVER) && _WIN32_WINNT >= 0x0500
	ofn.lStructSize = OPENFILENAME_SIZE_VERSION_400;
#else
	ofn.lStructSize = sizeof(ofn);
#endif
	ofn.lpstrFile = lpFile;
	ofn.nMaxFile = MAX_PATH;
	ofn.lpstrCustomFilter = lpCustomFilter;
	ofn.nMaxCustFilter = 42;
	
	if (lua_istable(L, 2)) {
		lua_getfield(L, 2, "Filter");
		ofn.lpstrFilter = luaL_optstring(L, -1, NULL);
		
		lua_getfield(L, 2, "CustomFilter");
		/* reserve last two NULL, 42 - 2 = 40 */
		strncpy(ofn.lpstrCustomFilter, luaL_optstring(L, -1, ""), 40);
		lua_pop(L, 1);
		
		lua_getfield(L, 2, "FilterIndex");
		ofn.nFilterIndex = luaL_optlong(L, -1, 1);
		lua_pop(L, 1);
		
		lua_getfield(L, 2, "File");
		strncpy(ofn.lpstrFile, luaL_optstring(L, -1, ""), MAX_PATH - 1);
		lua_pop(L, 1);
		
		lua_getfield(L, 2, "InitialDir");
		ofn.lpstrInitialDir = luaL_optstring(L, -1, NULL);
		
		lua_getfield(L, 2, "Title");
		ofn.lpstrTitle = luaL_optstring(L, -1, NULL);
		
		lua_getfield(L, 2, "Flags");
		ofn.Flags = luaL_optlong(L, -1, 0);
		lua_pop(L, 1);
		
		lua_getfield(L, 2, "DefExt");
		ofn.lpstrDefExt = luaL_optstring(L, -1, NULL);
	}
	
	ofn.Flags &= ~(OFN_ENABLEHOOK | OFN_ENABLETEMPLATE |
	               OFN_ENABLETEMPLATEHANDLE | OFN_SHAREAWARE);
	
	if ((*GetFileName)(&ofn)) {
		lua_pushstring(L, ofn.lpstrFile);
	} else {
		lua_pushnil(L);
	}
	
	lua_newtable(L);
	
	lua_pushstring(L, ofn.lpstrFilter);
	lua_setfield(L, -2, "Filter");
	
	lua_pushstring(L, ofn.lpstrCustomFilter);
	lua_setfield(L, -2, "CustomFilter");
	
	lua_pushinteger(L, ofn.nFilterIndex);
	lua_setfield(L, -2, "FilterIndex");
	
	lua_pushstring(L, ofn.lpstrFile);
	lua_setfield(L, -2, "File");
	
	lua_pushstring(L, ofn.lpstrInitialDir);
	lua_setfield(L, -2, "InitialDir");
	
	lua_pushstring(L, ofn.lpstrTitle);
	lua_setfield(L, -2, "Title");
	
	lua_pushinteger(L, ofn.Flags);
	lua_setfield(L, -2, "Flags");
	
	lua_pushinteger(L, ofn.nFileOffset);
	lua_setfield(L, -2, "FileOffset");
	
	lua_pushinteger(L, ofn.nFileExtension);
	lua_setfield(L, -2, "nFileExtension");
	
	lua_pushstring(L, ofn.lpstrDefExt);
	lua_setfield(L, -2, "DefExt");
	
	return 2;
}

static int L_OpenFileName(lua_State *L)
{
	return openfilename(L, GetOpenFileName);
}

static int L_SaveFileName(lua_State *L)
{
	return openfilename(L, GetSaveFileName);
}


/**
   SelectDirectory
   
   show the folder select dialog by SHBrowseForFolder Win32 API.
   
      name = SelectDirectory(hwnd [, tbl])
   
   hwnd  : integer : Window Handle of owner windows
   tbl   : table   : 
   
   If user selects cancel, name will be nil.
   
   members of tbl
   Root   : string  : root folder of tree
   Folder : string  : initial selected folder
   Title  : string  : dialog title
*/
static int L_SelectDirectory(lua_State *L)
{
	HWND hwnd;
	char lpRoot[MAX_PATH] = {0};
	char lpFolder[MAX_PATH] = {0};
	const char *title = "K-Meleon";
	
	hwnd = (HWND) luaL_checklong(L, 1);
	if (lua_istable(L, 2)) {
		lua_getfield(L, 2, "Root");
		lstrcpyn(lpRoot, luaL_optstring(L, -1, ""), MAX_PATH);
		lua_pop(L, 1);
		
		lua_getfield(L, 2, "Folder");
		lstrcpyn(lpFolder, luaL_optstring(L, -1, ""), MAX_PATH);
		lua_pop(L, 1);
		
		lua_getfield(L, 2, "Title");
		title = luaL_optstring(L, -1, NULL);
	}
	
	if (SelectDirectory(hwnd, title, lpRoot, lpFolder)) {
		lua_pushstring(L, lpFolder);
		return 1;
	}
	return 0;
}


luaL_Reg w32cd_funcs[] = {
	/* w32cd */
	{"ChooseColor",     L_ChooseColor},
	{"ChooseFont",      L_ChooseFont},
	{"GetOpenFileName", L_OpenFileName},
	{"GetSaveFileName", L_SaveFileName},
	{"SelectDirectory", L_SelectDirectory},
	/* end */
	{NULL, 0},
};


__declspec(dllexport) int luaopen_w32cd(lua_State *L)
{
	int i;
	const char *name;
	
	name = luaL_optstring(L, -1, LIBNAME);
	if (*name == '\0') {
		name = LIBNAME;
	}
	
	/* define module */
	luaL_register(L, name, w32cd_funcs);
	for (i = 0; w32cd_consts[i].name != NULL; i++) {
		lua_pushnumber(L, w32cd_consts[i].value);
		lua_setfield(L, -2, w32cd_consts[i].name);
	}
	return 1;
}
