#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "array.h"

ByteArray ByteArray_null = {NULL, 0, 0};

void ByteArray_Initialize(ByteArray *s)
{
	*s = ByteArray_null;
}

void ByteArray_Finalize(ByteArray *s)
{
	free(s->ptr);
	ByteArray_Initialize(s);
}

int ByteArray_Resize(ByteArray *s, size_t size)
{
	char *p;
	
	if (s->capacity >= size) return 0;
	p = realloc(s->ptr, size);
	if (!p && size > 0) return 1;
	s->ptr = p;
	s->capacity = size;
	return 0;
}

int ByteArray_Reserve(ByteArray *s, size_t size)
{
	return ByteArray_Resize(s, s->length + size);
}

int ByteArray_Append(ByteArray *s, char c)
{
	if (ByteArray_Reserve(s, 1)) return 1;
	s->ptr[s->length ++] = c;
	return 0;
}

int ByteArray_AppendBytes(ByteArray *s, const char *ptr, size_t size)
{
	if (ByteArray_Reserve(s, size)) return 1;
	memcpy(s->ptr + s->length, ptr, size);
	s->length += size;
	return 0;
}

int ByteArray_Padding(ByteArray *s, char c, size_t size)
{
	size_t n;
	
	n = size - s->length % size;
	if (n < size) {
		return ByteArray_AppendPadding(s, c, n);
	}
	return 0;
}

int ByteArray_AppendPadding(ByteArray *s, char c, size_t size)
{
	int i;
	
	if (ByteArray_Reserve(s, size)) return 1;
	for (i = 0; i < size; i ++) {
		s->ptr[s->length ++] = c;
	}
	return 0;
}
