#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include <string.h>

#include <stdio.h>

#include "array.h"
#include "list.h"
#include "dialog.h"

#define CHILDSTYLE (WS_CHILD | WS_VISIBLE | WS_DISABLED | WS_CAPTION |\
                    WS_BORDER | WS_VSCROLL | WS_HSCROLL | WS_GROUP |\
                    WS_TABSTOP)

static int AddStringA(ByteArray *a, LPCSTR str)
{
	int length, size;
	
	length = MultiByteToWideChar(CP_ACP, 0, str, -1, NULL, 0);
	if (length == 0) {
		return ByteArray_Padding(a, 0, sizeof(WCHAR));
	}
	size = length * sizeof(WCHAR);
	if (ByteArray_Reserve(a, size)) {
		return 1;
	}
	length = MultiByteToWideChar(CP_ACP, 0, str, -1,
	                             (LPWSTR) (a->ptr + a->length), length);
	if (length == 0) return 1;
	a->length += size;
	return 0;
}

static int AddStringW(ByteArray *a, LPCWSTR str)
{
	int length;
	
	if (! str) {
		return ByteArray_Padding(a, 0, sizeof(WCHAR));
	}
	length = lstrlenW(str) * sizeof(WCHAR) + sizeof(WCHAR);
	return ByteArray_AppendBytes(a, (char *) str, length);
}

#ifdef UNICODE
#define AddString AddStringW
#else
#define AddString AddStringA
#endif

static int AddControl(DialogTpl *d, LPDLGITEMTEMPLATE dt,
	DialogItem *lpdi, LPCWSTR cls, LPCTSTR title)
{
	ByteArray *a = &d->tmpl, *i = &d->item;
	LPDLGTEMPLATE lpdt;
	
	dt->style |= WS_CHILD | WS_VISIBLE;
	
	dt->dwExtendedStyle &=
		WS_EX_DLGMODALFRAME | WS_EX_TRANSPARENT | WS_EX_WINDOWEDGE |
		WS_EX_CLIENTEDGE | WS_EX_RIGHT | WS_EX_LEFT | WS_EX_RTLREADING |
		WS_EX_LTRREADING | WS_EX_LEFTSCROLLBAR | WS_EX_RIGHTSCROLLBAR |
		WS_EX_STATICEDGE | WS_EX_OVERLAPPEDWINDOW;
	
	if (ByteArray_Padding(a, 0, sizeof(DWORD)) ||
	    ByteArray_AppendBytes(a, (char *) dt, sizeof(DLGITEMTEMPLATE)) ||
	    ByteArray_Padding(a, 0, sizeof(WORD)) ||
	    AddStringW(a, cls) ||
	    ByteArray_Padding(a, 0, sizeof(WORD)) ||
	    AddString(a, title) ||
	    ByteArray_Padding(a, 0, sizeof(WORD)) ||
	    ByteArray_AppendPadding(a, 0, sizeof(WORD)) ||
	    ByteArray_AppendBytes(i, (char *) lpdi, sizeof(DialogItem))) {
		return 1;
	}
	lpdt = (LPDLGTEMPLATE) a->ptr;
	lpdt->cdit ++;
	d->num ++;
	return 0;
}


ListType *DialogTpl_GetInitialData(DialogTpl *d, WORD id)
{
	DialogItem *di = (DialogItem *) d->item.ptr;
	int i;
	
	for (i = 0; i < d->num; i ++, di ++) {
		if (di->id == id) {
			return &di->initial;
		}
	}
	return NULL;
}


ListType *DialogTpl_GetResult(DialogTpl *d, WORD id)
{
	DialogItem *di = (DialogItem *) d->item.ptr;
	int i;
	
	for (i = 0; i < d->num; i ++, di ++) {
		if (di->id == id) {
			return &di->result;
		}
	}
	return NULL;
}


int DialogTpl_New(DialogTpl *d, DWORD style, DWORD dwExtendedStyle,
	short x, short y, short cx, short cy, LPCTSTR caption,
	LPCTSTR font, WORD point)
{
	ByteArray *tmpl = &d->tmpl, *item = &d->item;
	DLGTEMPLATE dt = {0};
	
	ByteArray_Initialize(tmpl);
	ByteArray_Initialize(item);
	d->num = 0;
	
	style &=
		/* */
		WS_POPUP | WS_VISIBLE | WS_CAPTION | WS_BORDER | WS_DLGFRAME |
		WS_VSCROLL | WS_HSCROLL | WS_SYSMENU | WS_THICKFRAME | WS_GROUP |
		WS_TABSTOP | WS_MINIMIZEBOX | WS_MAXIMIZEBOX | WS_TILED |
		WS_SIZEBOX | WS_TILEDWINDOW | WS_OVERLAPPEDWINDOW | WS_POPUPWINDOW |
		WS_CHILDWINDOW |
		/* */
		DS_ABSALIGN | DS_MODALFRAME |
		DS_NOIDLEMSG | DS_SETFOREGROUND | DS_3DLOOK | DS_FIXEDSYS |
		DS_CENTER | DS_CENTERMOUSE;
	style |= WS_VISIBLE;
	if (font != NULL && lstrlen(font) > 0 && point > 0) {
		style |= DS_SETFONT;
	}
	
	dwExtendedStyle &=
		WS_EX_DLGMODALFRAME | WS_EX_TOPMOST | WS_EX_TRANSPARENT |
		WS_EX_TOOLWINDOW | WS_EX_WINDOWEDGE | WS_EX_CLIENTEDGE |
		WS_EX_RIGHT | WS_EX_LEFT | WS_EX_RTLREADING | WS_EX_LTRREADING |
		WS_EX_LEFTSCROLLBAR | WS_EX_RIGHTSCROLLBAR | WS_EX_CONTROLPARENT |
		WS_EX_STATICEDGE | WS_EX_OVERLAPPEDWINDOW | WS_EX_PALETTEWINDOW;
	
	dt.style = style;
	dt.dwExtendedStyle = dwExtendedStyle;
	dt.x = x;
	dt.y = y;
	dt.cx = cx;
	dt.cy = cy;
	if (ByteArray_AppendBytes(tmpl, (char *) &dt, sizeof(DLGTEMPLATE)) ||
	    ByteArray_Padding(tmpl, 0, sizeof(WORD)) ||
	    ByteArray_AppendPadding(tmpl, 0, sizeof(WORD) + sizeof(WORD)) ||
	    AddString(tmpl, caption)) {
		ByteArray_Finalize(tmpl);
		ByteArray_Finalize(item);
		return 1;
	}
	if (style & DS_SETFONT) {
		if (ByteArray_AppendBytes(tmpl, (char *) &point, sizeof(WORD)) ||
		    AddString(tmpl, font)) {
			ByteArray_Finalize(tmpl);
			ByteArray_Finalize(item);
			return 1;
		}
	}
	return 0;
}

void DialogTpl_Free(DialogTpl *d)
{
	DialogItem *di = (DialogItem *) d->item.ptr;
	int i;
	
	for (i = 0; i < d->num; i ++, di ++) {
		ListType_Finalize(&di->initial);
		ListType_Finalize(&di->result);
	}
	ByteArray_Finalize(&d->tmpl);
	ByteArray_Finalize(&d->item);
	d->num = 0;
}


/*
   Common Handler
 */
static int dummy_handler(HWND hwnd, DialogItem *self) { return 0; }
static int dummy_cmd(HWND hwnd, int id, int code) { return FALSE; }

static int get_text(HWND hwnd, DialogItem *self)
{
	int size;
	LPTSTR ptr;
	
	size = SendMessage(hwnd, WM_GETTEXTLENGTH, 0, 0) + 1;
	ptr = ListType_AllocBuffer(&self->result, sizeof(TCHAR) * size);
	if (ptr) {
		SendMessage(hwnd, WM_GETTEXT, size, (LPARAM) ptr);
		ptr[size - 1] = TEXT('\0');
	}
	return 0;
}

static int set_text(HWND hwnd, DialogItem *self)
{
	ListType *t = NULL;
	List *l;
	char *ptr;
	
	l = ListType_First(&self->initial, &t);
	while (t) {
		if (! ListType_GetBuffer(t, &ptr) && ptr) {
			SendMessage(hwnd, WM_SETTEXT, 0, (LPARAM) ptr);
		}
		l = ListType_Next(l, &t);
	}
	return 0;
}


/*
   Button
 */
static int button_get(HWND hwnd, DialogItem *self)
{
	int state;
	state = SendMessage(hwnd, BM_GETCHECK, 0, 0);
	ListType_SetNumber(&self->result, state);
	return 0;
}

static int button_set(HWND hwnd, DialogItem *self)
{
	int state = 0;
	
	ListType_GetNumber(&self->initial, &state);
	SendMessage(hwnd, BM_SETCHECK, state, 0);
	return 0;
}

static int button_cmd(HWND hwnd, int id, int code)
{
	return code == BN_CLICKED;
}

int DialogTpl_AddButton(DialogTpl *d, DWORD style, DWORD dwExtendedStyle,
	short x, short y, short cx, short cy, WORD id, LPCTSTR caption)
{
	DialogItem di = {0};
	DLGITEMTEMPLATE dt = {0};
	
	switch (style & 0xf) {
	case BS_PUSHBUTTON:
	case BS_DEFPUSHBUTTON:
	case BS_AUTOCHECKBOX:
	case BS_AUTO3STATE:
	case BS_GROUPBOX:
	case BS_AUTORADIOBUTTON:
		break;
	case BS_CHECKBOX:
		style = (style & ~0xf) | BS_AUTOCHECKBOX;
		break;
	case BS_RADIOBUTTON:
		style = (style & ~0xf) | BS_AUTORADIOBUTTON;
		break;
	case BS_3STATE:
		style = (style & ~0xf) | BS_AUTO3STATE;
		break;
	default:
		style = (style & ~0xf) | BS_PUSHBUTTON;
		break;
	}
	
	style &= CHILDSTYLE | 0xf | BS_LEFTTEXT | BS_TEXT |
	         BS_LEFT | BS_RIGHT | BS_CENTER | BS_TOP | BS_BOTTOM |
	         BS_VCENTER | BS_PUSHLIKE | BS_MULTILINE | BS_FLAT |
	         BS_RIGHTBUTTON;
	
	dt.style = style;
	dt.dwExtendedStyle = dwExtendedStyle;
	dt.x = x;
	dt.y = y;
	dt.cx = cx;
	dt.cy = cy;
	dt.id = id;
	
	di.id = id;
	di.get = button_get;
	di.set = button_set;
	
	if ((style & BS_AUTOCHECKBOX) != BS_AUTOCHECKBOX &&
	    (style & BS_AUTO3STATE) != BS_AUTO3STATE &&
	    (style & BS_AUTORADIOBUTTON) != BS_AUTORADIOBUTTON) {
		di.cmd = button_cmd;
	} else {
		di.cmd = dummy_cmd;
	}
	return AddControl(d, &dt, &di, WC_BUTTONW, caption);
}


/*
   Static
 */

static int static_cmd(HWND hwnd, int id, int code)
{
	return code == STN_CLICKED || code == STN_DBLCLK;
}

int DialogTpl_AddStatic(DialogTpl *d, DWORD style, DWORD dwExtendedStyle,
	short x, short y, short cx, short cy, WORD id, LPCTSTR caption)
{
	DialogItem di = {0};
	DLGITEMTEMPLATE dt = {0};
	
	switch (style & SS_TYPEMASK) {
	case SS_LEFT:
	case SS_CENTER:
	case SS_RIGHT:
	case SS_BLACKRECT:
	case SS_GRAYRECT:
	case SS_WHITERECT:
	case SS_BLACKFRAME:
	case SS_GRAYFRAME:
	case SS_WHITEFRAME:
	case SS_SIMPLE:
	case SS_LEFTNOWORDWRAP:
	case SS_ETCHEDHORZ:
	case SS_ETCHEDVERT:
	case SS_ETCHEDFRAME:
		break;
	default:
		style = (style & ~SS_TYPEMASK) | SS_LEFT;
		break;
	}
	
	style &= CHILDSTYLE | SS_TYPEMASK | SS_NOPREFIX | 
             SS_NOTIFY | SS_RIGHTJUST | SS_SUNKEN | SS_ENDELLIPSIS |
	         SS_PATHELLIPSIS | SS_WORDELLIPSIS | SS_ELLIPSISMASK;
	
	dt.style = style;
	dt.dwExtendedStyle = dwExtendedStyle;
	dt.x = x;
	dt.y = y;
	dt.cx = cx;
	dt.cy = cy;
	dt.id = id;
	
	di.id = id;
	di.get = dummy_handler;
	di.set = set_text;
	di.cmd = static_cmd;
	return AddControl(d, &dt, &di, WC_STATICW, caption);
}


/*
   Edit
 */

int DialogTpl_AddEdit(DialogTpl *d, DWORD style, DWORD dwExtendedStyle,
	short x, short y, short cx, short cy, WORD id, LPCTSTR caption)
{
	DialogItem di = {0};
	DLGITEMTEMPLATE dt = {0};
	
	style &= CHILDSTYLE |
	         ES_AUTOHSCROLL | ES_AUTOVSCROLL | ES_CENTER | ES_LEFT |
	         ES_LOWERCASE | ES_MULTILINE | ES_NOHIDESEL | ES_NUMBER |
	         ES_OEMCONVERT | ES_PASSWORD | ES_READONLY | ES_RIGHT |
	         ES_UPPERCASE | ES_WANTRETURN;
	
	dt.style = style;
	dt.dwExtendedStyle = dwExtendedStyle;
	dt.x = x;
	dt.y = y;
	dt.cx = cx;
	dt.cy = cy;
	dt.id = id;
	
	di.id = id;
	di.get = get_text;
	di.set = set_text;
	di.cmd = dummy_cmd;
	return AddControl(d, &dt, &di, WC_EDITW, caption);
}


/*
   Listbox
 */

static int listbox_is_multiplesel(UINT style)
{
	return (style & LBS_MULTIPLESEL) || (style & LBS_EXTENDEDSEL);
}

static int listbox_get(HWND hwnd, DialogItem *self)
{
	UINT style;
	int i, n, *ptr = NULL;
	
	style = GetWindowLongPtr(hwnd, GWL_STYLE);
	/* single select */
	if (! listbox_is_multiplesel(style)) {
		n = SendMessage(hwnd, LB_GETCURSEL, 0, 0);
		if (n != LB_ERR && (style & LBS_SORT)) {
			n = SendMessage(hwnd, LB_GETITEMDATA, n, 0);
		}
		return ListType_SetNumber(&self->result, n);
	}
	/* multiple select */
	n = SendMessage(hwnd, LB_GETSELCOUNT, 0, 0);
	ptr = ListType_PushAllocBuffer(&self->result, sizeof(int) * (n + 1));
	if (! ptr) return 1;
	if (n > 0 &&
	    LB_ERR == SendMessage(hwnd, LB_GETSELITEMS, n, (LPARAM) ptr)) {
		ListType_Finalize(&self->result);
		return 1;
	}
	ptr[n] = -1;
	if (style & LBS_SORT) {
		for (i = 0; i < n; i ++) {
			ptr[i] = SendMessage(hwnd, LB_GETITEMDATA, ptr[i], 0);
		}
	}
	return 0;
}

static int listbox_set(HWND hwnd, DialogItem *self)
{
	ListType *t = NULL;
	List *l = NULL;
	UINT style;
	char *ptr = NULL;
	int i, c, sel = -1, *selp = NULL, *p;
	
	style = GetWindowLongPtr(hwnd, GWL_STYLE);
	l = ListType_First(&self->initial, &t);
	/* */
	if (t) {
		switch (t->type) {
		case NUMBER:
			ListType_GetNumber(t, &sel);
			break;
		case BUFFER:
			if (! ListType_GetBuffer(t, &selp) && selp) {
				sel = 0;
			}
			break;
		case LIST:
			return 1;
		}
		l = ListType_Next(l, &t);
	}
	/* */
	while (t) {
		if (! ListType_GetBuffer(t, &ptr) && ptr) {
			i = SendMessage(hwnd, LB_ADDSTRING, 0, (LPARAM) ptr);
			if (i == LB_ERR || i == LB_ERRSPACE) return 1;
			c = SendMessage(hwnd, LB_GETCOUNT, 0, 0) - 1;
			SendMessage(hwnd, LB_SETITEMDATA, i, (LPARAM) c);
			if (sel >= 0) {
				if (selp) {
					for (p = selp; p && *p >= 0; p ++) {
						if (*p == c) {
							if (listbox_is_multiplesel(style)) {
								SendMessage(hwnd, LB_SETSEL, TRUE, i);
							} else {
								SendMessage(hwnd, LB_SETCURSEL, i, 0);
							}
							break;
						}
					}
				} else if (c == sel) {
					if (listbox_is_multiplesel(style)) {
						SendMessage(hwnd, LB_SETSEL, TRUE, i);
					} else {
						SendMessage(hwnd, LB_SETCURSEL, i, 0);
					}
				}
			}
		}
		l = ListType_Next(l, &t);
	}
	return 0;
}

static int listbox_cmd(HWND hwnd, int id, int code)
{
	return code == LBN_DBLCLK;
}

int DialogTpl_AddListbox(DialogTpl *d, DWORD style, DWORD dwExtendedStyle,
	short x, short y, short cx, short cy, WORD id, LPCTSTR caption)
{
	DialogItem di = {0};
	DLGITEMTEMPLATE dt = {0};
	
	style &=
		CHILDSTYLE | LBS_NOTIFY | LBS_SORT | LBS_MULTIPLESEL |
		LBS_HASSTRINGS | LBS_USETABSTOPS | LBS_NOINTEGRALHEIGHT |
		LBS_MULTICOLUMN | LBS_EXTENDEDSEL | LBS_DISABLENOSCROLL |
		LBS_STANDARD;
	
	dt.style = style;
	dt.dwExtendedStyle = dwExtendedStyle;
	dt.x = x;
	dt.y = y;
	dt.cx = cx;
	dt.cy = cy;
	dt.id = id;
	
	di.id = id;
	di.get = listbox_get;
	di.set = listbox_set;
	di.cmd = listbox_cmd;
	return AddControl(d, &dt, &di, WC_LISTBOXW, caption);
}


/*
   Combobox
 */
static int combobox_set(HWND hwnd, DialogItem *self)
{
	ListType *t = NULL;
	List *l = NULL;
	char *ptr = NULL, *sel = NULL;
	int i;
	UINT style;
	
	style = GetWindowLongPtr(hwnd, GWL_STYLE);
	l = ListType_First(&self->initial, &t);
	if (t) {
		ListType_GetBuffer(t, &sel);
		if (sel && (style & 0xf) != CBS_DROPDOWNLIST) {
			SendMessage(hwnd, WM_SETTEXT, 0, (LPARAM) sel);
			sel = NULL;
		}
		l = ListType_Next(l, &t);
	}
	while (t) {
		if (! ListType_GetBuffer(t, &ptr) && ptr) {
			i = SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) ptr);
			if (i == CB_ERR || i == CB_ERRSPACE) return 1;
			if (sel && ! lstrcmp(ptr, sel)) {
				SendMessage(hwnd, CB_SETCURSEL, i, 0);
			}
		}
		l = ListType_Next(l, &t);
	}
	return 0;
}

int DialogTpl_AddCombobox(DialogTpl *d, DWORD style, DWORD dwExtendedStyle,
	short x, short y, short cx, short cy, WORD id, LPCTSTR caption)
{
	DialogItem di = {0};
	DLGITEMTEMPLATE dt = {0};
	
	style &=
		CHILDSTYLE | CBS_SIMPLE | CBS_DROPDOWN | CBS_DROPDOWNLIST |
		CBS_AUTOHSCROLL | CBS_OEMCONVERT | CBS_SORT |
		CBS_NOINTEGRALHEIGHT | CBS_DISABLENOSCROLL |
		CBS_UPPERCASE | CBS_LOWERCASE;
	
	dt.style = style;
	dt.dwExtendedStyle = dwExtendedStyle;
	dt.x = x;
	dt.y = y;
	dt.cx = cx;
	dt.cy = cy;
	dt.id = id;
	
	di.id = id;
	di.get = get_text;
	di.set = combobox_set;
	di.cmd = dummy_cmd;
	return AddControl(d, &dt, &di, WC_COMBOBOXW, caption);
}


/*
 */

BOOL InitializeItems(HWND hwnd, DialogTpl *lpdt)
{
	HWND hwndDlgItem;
	DialogItem *lpdi;
	int i;
	
	lpdi = (DialogItem *) lpdt->item.ptr;
	if (! lpdi) return FALSE;
	for (i = 0; i < lpdt->num; i ++, lpdi ++) {
		hwndDlgItem = GetDlgItem(hwnd, lpdi->id);
		if (lpdi->set(hwndDlgItem, lpdi)) {
			return FALSE;
		}
		ListType_Finalize(&lpdi->result);
	}
	return TRUE;
}

void GetResults(HWND hwnd, DialogTpl *lpdt)
{
	HWND hwndDlgItem;
	DialogItem *lpdi;
	int i;
	
	lpdi = (DialogItem *) lpdt->item.ptr;
	if (lpdi) {
		for (i = 0; i < lpdt->num; i ++, lpdi ++) {
			hwndDlgItem = GetDlgItem(hwnd, lpdi->id);
			lpdi->get(hwndDlgItem, lpdi);
		}
	}
}


/*
 */

struct _dialog_param {
	LPCTSTR lpCaption;
	DialogTpl *lpdt;
};

static BOOL OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lParam)
{
	struct _dialog_param *lpParam;
	DialogTpl *lpdt;
	
	lpParam = (struct _dialog_param *) lParam;
	if (lpParam) {
		if (lpParam->lpCaption) {
			SendMessage(hwnd, WM_SETTEXT, 0, (LPARAM) lpParam->lpCaption);
		}
		lpdt = lpParam->lpdt;
		SetWindowLongPtr(hwnd, DWLP_USER, (LONG_PTR) lpdt);
		if ((LPVOID) GetWindowLongPtr(hwnd, DWLP_USER) != lpdt ||
			! InitializeItems(hwnd, lpdt)) {
			EndDialog(hwnd, -1);
			return TRUE;
		}
	}
	return TRUE;
}

static void OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
	DialogTpl *lpdt;
	DialogItem *lpdi;
	int i;
	
	lpdt = (LPVOID) GetWindowLongPtr(hwnd, DWLP_USER);
	if (lpdt) {
		lpdi = (DialogItem *) lpdt->item.ptr;
		if (lpdi) {
			for (i = 0; i < lpdt->num; i ++, lpdi ++) {
				if (lpdi->id == id &&
				    lpdi->cmd(hwndCtl, id, codeNotify)) {
					break;
				}
			}
			if (i < lpdt->num) {
				GetResults(hwnd, lpdt);
				EndDialog(hwnd, id);
			}
		}
	}
}

BOOL CALLBACK
DlgProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {
	case WM_INITDIALOG:
		OnInitDialog(hwnd, (HWND) wParam, lParam);
		return TRUE;
	case WM_COMMAND:
		OnCommand(hwnd, LOWORD(wParam), (HWND) lParam, HIWORD(wParam));
		return TRUE;
	default:
		return FALSE;
	}
}


int DialogTpl_ShowModal(DialogTpl *lpdt, LPCTSTR lpCaption,
	HANDLE hInstance, HWND hwndParent)
{
	struct _dialog_param param;
	
	param.lpCaption = lpCaption;
	param.lpdt = lpdt;
	return DialogBoxIndirectParam(hInstance,
		(LPCDLGTEMPLATE) lpdt->tmpl.ptr, hwndParent, DlgProc,
		(LPARAM) &param);
}

HWND DialogTpl_Show(DialogTpl *lpdt, LPCTSTR lpCaption,
	HANDLE hInstance, HWND hwndParent)
{
	struct _dialog_param param;
	
	param.lpCaption = lpCaption;
	param.lpdt = lpdt;
	return CreateDialogIndirectParam(hInstance,
		(LPCDLGTEMPLATE) lpdt->tmpl.ptr, hwndParent, DlgProc,
		(LPARAM) &param);
}
