--[[

	dialog demo
	
	Please eval next line with eval dialog.
		require "dialog_demo"; dialog_demo()

	+-------------------------------------+
	| IDBUTTON IDCHECK1 ID3STATE IDCHECK2 |
	| +-IDGROUP--+                        |
	| | IDRADIO1 |   IDEDIT1              |
	| | IDRADIO2 |   IDEDIT2     IDEDIT4  |
	| | IDRADIO3 |   IDEDIT3              |
	| +----------+                        |
	|                                     |
	|    IDLIST1     IDLIST2     IDLIST3  |
	|                                     |
	|   IDCONBO1     IDCONBO2   IDSTATIC1 |
	|                IDCONBO3   IDSTATIC2 |
	+-------------------------------------+

]]

require "dialog"

local d = create_dialog({
	width = 180,         -- width
	height = 165,        -- height
	caption = "dialog",  -- caption
	style = w32dlg.DS_CENTER,
	
	--
	-- Button
	--
	
	-- simple button
	-- When button clicked, dialog will close and return ID.
	{
		type = "button",    -- control type
		style = w32dlg.BS_DEFPUSHBUTTON,
		x = 5,              -- x position
		y = 5,              -- y position
		width = 30,         -- width
		height = 10,        -- height
		id = "IDBUTTON",    -- ID
		caption = "OK"      -- caption
	},
	
	-- checkbox
	-- state is w32dlg.BST_UNCHECKED or w32dlg.BST_CHECKED
	{
		type = "button",    -- control type
		style = w32dlg.BS_CHECKBOX,
		x = 40,             -- x position
		y = 5,              -- y position
		width = 40,         -- width
		height = 10,        -- height
		id = "IDCHECK1",    -- ID
		caption = "CHECK1"  -- caption
	},
	
	-- 3 state checkbox
	-- state is BST_UNCHECKED, BST_CHECKED, BST_INDETERMINATE
	{
		type = "button",    -- control type
		style = w32dlg.BS_3STATE,
		x = 85,             -- x position
		y = 5,              -- y position
		width = 40,         -- width
		height = 10,        -- height
		id = "ID3STATE",    -- ID
		caption = "3STATE"  -- caption
	},
	
	-- push like checkbox
	-- state is BST_UNCHECKED or BST_CHECKED
	{
		type = "button",    -- control type
		style = w32dlg.BS_CHECKBOX + w32dlg.BS_PUSHLIKE,
		x = 130,            -- x position
		y = 5,              -- y position
		width = 40,         -- width
		height = 10,        -- height
		id = "IDCHECK2",    -- ID
		caption = "CHECK2"  -- caption
	},
	
	-- group box
	{
		type = "button",    -- control type
		style = w32dlg.BS_GROUPBOX,
		x = 5,              -- x position
		y = 20,             -- y position
		width = 50,         -- width
		height = 45,        -- height
		id = "IDGROUP",     -- ID
		caption = "RADIO"   -- caption
	},
	
	-- radio button
	-- grouped by w32dlg.WS_GROUP
	-- state is w32dlg.BST_UNCHECKED or w32dlg.BST_CHECKED
	{
		type = "button",    -- control type
		style = w32dlg.BS_RADIOBUTTON + w32dlg.WS_GROUP,
		x = 10,             -- x position
		y = 30,             -- y position
		width = 40,         -- width
		height = 10,        -- height
		id = "IDRADIO1",    -- ID
		caption = "RADIO1"  -- caption
	},
	
	-- radio button
	{
		type = "button",    -- control type
		style = w32dlg.BS_RADIOBUTTON,
		x = 10,             -- x position
		y = 40,             -- y position
		width = 40,         -- width
		height = 10,        -- height
		id = "IDRADIO2",    -- ID
		caption = "RADIO2"  -- caption
	},
	-- radio button
	{
		type = "button",    -- control type
		style = w32dlg.BS_RADIOBUTTON,
		x = 10,             -- x position
		y = 50,             -- y position
		width = 40,         -- width
		height = 10,        -- height
		id = "IDRADIO3",    -- ID
		caption = "RADIO3"  -- caption
	},
	
	--
	-- editbox
	--
	
	-- simple
	{
		type = "edit",     -- control type
		style = w32dlg.WS_BORDER,
		x = 60,            -- x position
		y = 20,            -- y position
		width = 50,        -- width
		height = 10,       -- height
		id = "IDEDIT1",    -- ID
		caption = "EDIT"   -- caption
	},
	
	-- password
	{
		type = "edit",     -- control type
		style = w32dlg.WS_BORDER + w32dlg.ES_PASSWORD,
		x = 60,            -- x position
		y = 35,            -- y position
		width = 50,        -- width
		height = 10,       -- height
		id = "IDEDIT2",    -- ID
		caption = "EDIT"   -- caption
	},
	
	-- number only
	{
		type = "edit",     -- control type
		style = w32dlg.WS_BORDER + w32dlg.ES_NUMBER,
		x = 60,            -- x position
		y = 50,            -- y position
		width = 50,        -- width
		height = 10,       -- height
		id = "IDEDIT3",    -- ID
		caption = "0000"   -- caption
	},
	
	-- multiline
	{
		type = "edit",          -- control type
		style = w32dlg.WS_BORDER +
		        w32dlg.WS_VSCROLL + w32dlg.WS_HSCROLL +
		        w32dlg.ES_MULTILINE + w32dlg.ES_AUTOVSCROLL +
		        w32dlg.ES_AUTOHSCROLL + w32dlg.ES_WANTRETURN,
		x = 115,                -- x position
		y = 20,                 -- y position
		width = 55,             -- width
		height = 40,            -- height
		id = "IDEDIT4",         -- ID
		caption = "MULTILINE"   -- caption
	},
	
	--
	-- Listbox
	--
	
	-- When a item is double clicked, dialog will close and returns ID.
	{
		type = "listbox",          -- control type
		style = w32dlg.WS_BORDER +
		        w32dlg.WS_VSCROLL + w32dlg.WS_HSCROLL +
		        w32dlg.LBS_NOTIFY, -- enable double click
		x = 5,                  -- x position
		y = 70,                 -- y position
		width = 50,             -- width
		height = 40,            -- height
		id = "IDLIST1",         -- ID
	},
	
	-- Multi select
	{
		type = "listbox",          -- control type
		style = w32dlg.WS_BORDER +
		        w32dlg.WS_VSCROLL + w32dlg.WS_HSCROLL +
		        w32dlg.LBS_MULTIPLESEL,
		x = 60,                  -- x position
		y = 70,                 -- y position
		width = 50,             -- width
		height = 40,            -- height
		id = "IDLIST2",         -- ID
	},
	
	-- Multi select ctrl+click or shift+drag
	-- sort
	{
		type = "listbox",          -- control type
		style = w32dlg.WS_BORDER +
		        w32dlg.WS_VSCROLL + w32dlg.WS_HSCROLL +
		        w32dlg.LBS_EXTENDEDSEL + w32dlg.LBS_SORT,
		x = 115,                  -- x position
		y = 70,                 -- y position
		width = 50,             -- width
		height = 40,            -- height
		id = "IDLIST3",         -- ID
	},
	
	-- 
	-- Combobox
	-- 
	
	-- simple
	-- edit and list
	{
		type = "combobox",          -- control type
		style = w32dlg.WS_BORDER +
		        w32dlg.WS_VSCROLL + w32dlg.WS_HSCROLL +
		        w32dlg.CBS_SIMPLE,
		x = 5,                  -- x position
		y = 110,                -- y position
		width = 50,             -- width
		height = 40,            -- height
		id = "IDCOMBO1",         -- ID
	},
	
	-- drop down
	-- edit and popup list
	{
		type = "combobox",          -- control type
		style = w32dlg.WS_BORDER +
		        w32dlg.WS_VSCROLL + w32dlg.WS_HSCROLL +
		        w32dlg.CBS_DROPDOWN,
		x = 60,                  -- x position
		y = 110,                -- y position
		width = 50,             -- width
		height = 40,            -- height
		id = "IDCOMBO2",         -- ID
	},
	
	-- drop down list
	-- no editable, popup list only
	{
		type = "combobox",          -- control type
		style = w32dlg.WS_BORDER +
		        w32dlg.WS_VSCROLL + w32dlg.WS_HSCROLL +
		        w32dlg.CBS_DROPDOWNLIST,
		x = 60,                  -- x position
		y = 125,                -- y position
		width = 50,             -- width
		height = 40,            -- height
		id = "IDCOMBO3",         -- ID
	},
	
	--
	-- Static
	--
	
	-- simple
	{
		type = "static",        -- control type
		x = 120,                 -- x position
		y = 110,                -- y position
		width = 50,             -- width
		height = 10,            -- height
		id = "IDSTATIC1",       -- ID
		caption = "static"
	},
	
	-- clickable
	{
		type = "static",        -- control type
		style = w32dlg.SS_NOTIFY,
		x = 120,                -- x position
		y = 125,                -- y position
		width = 50,             -- width
		height = 10,            -- height
		id = "IDSTATIC2",       -- ID
		caption = "click!"
	},
})


local function print_result(r, rr)
	local k, v, t, l
	
	l = {"Button",
	     "IDCHECK1", "ID3STATE", "IDCHECK2",
	     "IDRADIO1", "IDRADIO2", "IDRADIO3",
	     "Edit",
	     "IDEDIT1", "IDEDIT2", "IDEDIT3", "IDEDIT4",
	     "Listbox",
	     "IDLIST1", "IDLIST2", "IDLIST3",
	     "Combobox",
	     "IDCOMBO1", "IDCOMBO2", "IDCOMBO3",}
	
	t = "ID\t:\t" .. r .. "\r\n"
	for k, v in ipairs(l) do
		if rr[v] then
			if type(rr[v]) == "number" or
			   type(rr[v]) == "string" then
				t = t .. v .. " :\t" .. rr[v] .. "\r\n"
			elseif type(rr[v]) == "table" then
				t = t .. v .. " :\t" ..
				    table.concat(rr[v], " ") .. "\r\n"
			end
		else
			t = t .. v .. "\r\n"
		end
	end
	if alert then
		alert(t)
	else
		print(t)
	end
end

function dialog_demo()
	local r, rr, t, k, v, m
	
	m = {"January", "February", "March", "April", "May", "June",
	     "July", "August", "September", "October", "November", "December"}
	r, rr = d(nil, {
		IDLIST1 = {
			{1},  -- initial selection
			m,    -- items
		},
		IDLIST2 = {nil, m},
		IDLIST3 = {nil, m},
		IDCOMBO1 = {
			nil,  -- initial string, nil is default (empty)
			m,    -- items
		},
		IDCOMBO2 = {nil, m},
		IDCOMBO3 = {nil, m},
	})
	print_result(r, rr)
	r, rr = d("dialog demo 2nd", {
		IDCHECK1 = rr.IDCHECK1,
		IDCHECK2 = rr.IDCHECK2,
		ID3STATE = rr.ID3STATE,
		IDRADIO1 = rr.IDRADIO1,
		IDRADIO2 = rr.IDRADIO2,
		IDRADIO3 = rr.IDRADIO3,
		IDEDIT1  = rr.IDEDIT1,
		IDEDIT2  = rr.IDEDIT2,
		IDEDIT3  = rr.IDEDIT3,
		IDEDIT4  = rr.IDEDIT4,
		IDLIST1 = {rr.IDLIST1, m},
		IDLIST2 = {rr.IDLIST2, m},
		IDLIST3 = {rr.IDLIST3, m},
		IDCOMBO1 = {rr.IDCOMBO1, m},
		IDCOMBO2 = {rr.IDCOMBO2, m},
		IDCOMBO3 = {rr.IDCOMBO3, m},
	})
	print_result(r, rr)
end

if not km then
	dialog_demo()
end
