--[[
	trashy converter of dialog resource script to lua table 
]]

local function get_token(s, i)
	s, i = string.match(s, "[ ,\t\r\n]*([^ ,\t\r\n]+)()", i)
	return s, i
end

local function get_string(s, i)
	s, i = string.match(s, "[^\"]*\"([^\"]*)\"()", i)
	-- todo: escape double quotation by twice
	return s, i
end

local function get_integer(s, i)
	s, i = string.match(s, "[^%d]*(%d+)()", i)
	return s, i
end

local function is_dialog_statement(tok)
	return tok == "CAPTION" or tok == "CHARACTERISTICS" or
	       tok == "CLASS" or tok == "EXSTYLE" or
	       tok == "FONT" or tok == "LANGUAGE" or
	       tok == "STYLE" or tok == "VERSION" or
	       tok == "BEGIN" or tok == "{"
end

local function is_control_statement(tok)
	return tok == "AUTO3STATE" or tok == "AUTOCHECKBOX" or
	       tok == "AUTORADIOBUTTON" or tok == "CHECKBOX" or
	       tok == "COMBOBOX" or tok == "CONTROL" or
	       tok == "CTEXT" or tok == "DEFPUSHBUTTON" or
	       tok == "EDITTEXT" or tok == "GROUPBOX" or
	       tok == "ICON" or tok == "LISTBOX" or
	       tok == "LTEXT" or tok == "PUSHBOX" or
	       tok == "PUSHBUTTON" or tok == "RADIOBUTTON" or
	       tok == "RTEXT" or tok == "SCROLLBAR" or
	       tok == "STATE3" or tok == "END" or tok == "}"
end

local function get_style(tpl, idx)
	local tok, i, style
	tok, i = get_token(tpl, idx)
	if is_dialog_statement(tok) or is_control_statement(tok) then
		return nil, idx
	end
	idx = i
	style = "w32dlg." .. tok
	while true do
		tok, i = get_token(tpl, idx)
		if not tok or tok ~= "|" then
			return style, idx
		end
		idx = i
		tok, idx = get_token(tpl, idx)
		style = style .. "+w32dlg." .. tok
	end
end

local function get_font(tpl, idx)
	local point, family, tok, i
	point, idx = get_integer(tpl, idx)
	family, idx = get_string(tpl, idx)
	while true do
		tok, i = get_token(tpl, idx)
		if not tok or is_dialog_statement(tok) then
			break
		end
		idx = i
	end
	return "font=\""..family.."\",point="..point..",\n", idx
end

local function get_metrics(tpl, idx)
	local x, y, cx, cy
	x, idx = get_integer(tpl, idx)
	y, idx = get_integer(tpl, idx)
	cx, idx = get_integer(tpl, idx)
	cy, idx = get_integer(tpl, idx)
	return "x="..x..",y="..y..",width="..cx..",height="..cy..",\n", idx
end

local function parse_class(tpl, idx, style)
	local tok, result
	tok, idx = get_token(tpl, idx)
	result = "id=\""..tok.."\",\n"
	tok, idx = get_metrics(tpl, idx)
	result = result .. tok
	tok, idx = get_style(tpl, idx)
	if tok or style then
		result = result .. "style=" .. (tok or style) .. ",\n"
	end
	tok, idx = get_style(tpl, idx)
	if tok then
		result = result .. "estyle=" .. tok .. ",\n"
	end
	return result, idx
end

local function parse_class_text(tpl, idx, style)
	local tok, result
	tok, idx = get_string(tpl, idx)
	result = "caption=\"" .. tok .. "\",\n"
	tok, idx = parse_class(tpl, idx, style)
	result = result .. tok
	return result, idx
end

local function parse_control(tpl, idx, dstyle)
	local tok, i, result
	local caption, id, typ, class, style, metrics, estyle
	caption, idx = get_string(tpl, idx)
	id, idx = get_token(tpl, idx)
	tok, idx = get_string(tpl, idx)
	typ = string.lower(tok)
	style, idx = get_style(tpl, idx)
	metrics, idx = get_metrics(tpl, idx)
	tok, idx = get_style(tpl, idx)
	if tok then
		estyle = "estyle=\""..tok.."\"\n"
	else
		estyle = ""
	end
	return "{\n"..
	       "type=\""..typ.."\",\n"..
	       "caption=\"" .. caption .. "\",\n"..
	       "id=\""..id.."\",\n"..
	       "style="..style..",\n"..
	       metrics..estyle.."},\n", idx
end

function convert_dialog_template(tpl)
	local idx = 0
	local tok, i, result
	while true do
		tok, idx = get_token(tpl, idx)
		if not tok then
			return
		end
		if tok == "DIALOG" or tok == "DIALOGEX" then
			break
		end
	end
	tok, i = get_token(tpl, idx)
	if tok == "DISCARDABLE" then
		idx = i
	end
	tok, idx = get_metrics(tpl, idx)
	result = "{\n" .. tok
	while true do
		tok, idx = get_token(tpl, idx)
		if not tok or tok == "END" or tok == "}" then
			return result .. "}\n"
		elseif tok == "CAPTION" then
			tok, idx = get_string(tpl, idx)
			result = result .. "caption=\"" .. tok .. "\",\n"
		elseif tok == "STYLE" then
			tok, idx = get_style(tpl, idx)
			result = result .. "style=" .. tok .. ",\n"
		elseif tok == "EXSTYLE" then
			tok, idx = get_style(tpl, idx)
			result = result .. "estyle=" .. tok .. ",\n"
		elseif tok == "FONT" then
			tok, idx = get_font(tpl, idx)
			result = result .. tok
		elseif tok == "CHARACTERISTICS" or tok == "CLASS" or
		       tok == "MENU" or tok == "VERSION" then
			tok, idx = get_token(tpl, idx)
			-- ignore
		elseif tok == "BEGIN" or tok == "{" then
			break;
		end
	end
	while true do
		tok, idx = get_token(tpl, idx)
		if not tok or tok == "END" or tok == "}" then
			break;
		elseif tok == "AUTO3STATE" or tok == "STATE3" then
			tok, idx = parse_class_text(tpl, idx,
			           "w32dlg.AUTO3STATE+w32dlg.WS_TABSTOP")
			result = result.."{\ntype=\"button\",\n"..tok.."},\n"
		elseif tok == "AUTOCHECKBOX" or tok == "CHECKBOX" then
			tok, idx = parse_class_text(tpl, idx,
			           "w32dlg.BS_AUTOCHECKBOX+w32dlg.WS_TABSTOP")
			result = result.."{\ntype=\"button\",\n"..tok.."},\n"
		elseif tok == "AUTORADIOBUTTON" or tok == "RADIOBUTTON" then
			tok, idx = parse_class_text(tpl, idx,
			           "w32dlg.BS_AUTORADIOBUTTON+w32dlg.WS_TABSTOP")
			result = result.."{\ntype=\"button\",\n"..tok.."},\n"
		elseif tok == "COMBOBOX" then
			tok, idx = parse_class(tpl, idx,
			           "w32dlg.CBS_SIMPLE+w32dlg.WS_TABSTOP")
			result = result.."{\ntype=\"combobox\",\n"..tok.."},\n"
		elseif tok == "CONTROL" then
			tok, idx = parse_control(tpl, idx)
			result = result..tok
		elseif tok == "CTEXT" then
			tok, idx = parse_class_text(tpl, idx,
			           "w32dlg.SS_CENTER+w32dlg.WS_GROUP")
			result = result.."{\ntype=\"static\",\n"..tok.."},\n"
		elseif tok == "DEFPUSHBUTTON" then
			tok, idx = parse_class_text(tpl, idx,
			           "w32dlg.BS_DEFPUSHBUTTON+w32dlg.WS_TABSTOP")
			result = result.."{\ntype=\"button\",\n"..tok.."},\n"
		elseif tok == "EDITTEXT" then
			tok, idx = parse_class(tpl, idx,
			           "w32dlg.ES_LEFT+w32dlg.WS_BORDER+w32dlg.WS_TABSTOP")
			result = result.."{\ntype=\"edit\",\n"..tok.."},\n"
		elseif tok == "GROUPBOX" then
			tok, idx = parse_class_text(tpl, idx,"w32dlg.BS_GROUPBOX")
			result = result.."{\ntype=\"button\",\n"..tok.."},\n"
		elseif tok == "ICON" then
			tok, idx = parse_class_text(tpl, idx, "")
			-- ignore
		elseif tok == "LISTBOX" then
			tok, idx = parse_class(tpl, idx,
			           "w32dlg.LBS_NOTIFY+w32dlg.WS_BORDER")
			result = result.."{\ntype=\"listbox\",\n"..tok.."},\n"
		elseif tok == "LTEXT" then
			tok, idx = parse_class_text(tpl, idx,
			           "w32dlg.SS_LEFT+w32dlg.WS_GROUP")
			result = result.."{\ntype=\"static\",\n"..tok.."},\n"
		elseif tok == "PUSHBOX" then
			tok, idx = parse_class_text(tpl, idx, "")
			-- ignore
		elseif tok == "PUSHBUTTON" then
			tok, idx = parse_class_text(tpl, idx,
			           "w32dlg.BS_PUSHBUTTON+w32dlg.WS_TABSTOP")
			result = result.."{\ntype=\"button\",\n"..tok.."},\n"
		elseif tok == "RTEXT" then
			tok, idx = parse_class_text(tpl, idx,
			           "w32dlg.SS_RIGHT+w32dlg.WS_GROUP")
			result = result.."{\ntype=\"static\",\n"..tok.."},\n"
		elseif tok == "SCROLLBAR" then
			tok, idx = parse_class(tpl, idx, "")
			-- ignore
		end
	end
	return result .. "}"
end
