#include <stdlib.h>
#include <string.h>

#include "list.h"


static void List_Set(List *l, const ListType *data, List *next)
{
	const static ListType z = {0};
	
	l->data = (data) ? *data : z;
	l->next = next;
}

static List *List_New(const ListType *data, List *next)
{
	List *l = NULL;
	
	l = malloc(sizeof(List));
	if (!l) return NULL;
	List_Set(l, data, next);
	return l;
}

static void List_Delete(List *s)
{
	List *p;
	
	while (s) {
		ListType_Finalize(&s->data);
		p = s->next;
		free(s);
		s = p;
	}
}

static List *List_Reverse(List *l)
{
	List *n, *p = NULL;
	
	while (l) {
		n = List_Next(l);
		l->next = p;
		p = l;
		l = n;
	}
	return p;
}

static int ListType_Push(ListType *a, const ListType *b)
{
	List *la, *lb;
	
	if (!a) return 1;
	if ((a->type == LIST || a->type == BUFFER) && a->u.l == NULL) {
		*a = *b;
		return 0;
	}
	lb = List_New(b, NULL);
	if (!lb) return 1;
	if (a->type != LIST) {
		la = List_New(a, NULL);
		if (!la) {
			free(lb);
			return 1;
		}
		a->type = LIST;
		lb->next = la;
		a->u.l = lb;
	} else {
		lb->next = a->u.l;
		a->u.l = lb;
	}
	return 0;
}

/*
 */

int ListType_SetNumber(ListType *t, int n)
{
	if (!t) return 1;
	ListType_Finalize(t);
	t->type = NUMBER;
	t->u.n = n;
	return 0;
}

int ListType_PushNumber(ListType *t, int n)
{
	ListType b = {0};
	
	return ListType_SetNumber(&b, n) ||
	       ListType_Push(t, &b);
}

int ListType_GetNumber(const ListType *t, int *n)
{
	if (!t || !n || t->type != NUMBER) return 1;
	*n = t->u.n;
	return 0;
}

void *ListType_AllocBuffer(ListType *t, size_t size)
{
	void *p;
	
	if (!t) return NULL;
	p = malloc(size);
	if (p) {
		ListType_Finalize(t);
		t->type = BUFFER;
		t->u.b = p;
	}
	return p;
}

int ListType_CopyBuffer(ListType *t, const void *ptr, size_t size)
{
	void *p;
	
	if (! ptr || ! (p = ListType_AllocBuffer(t, size))) return 1;
	memcpy(p, ptr, size);
	return 0;
}

void *ListType_PushAllocBuffer(ListType *t, size_t size)
{
	ListType b = {0};
	void *p;
	
	p = ListType_AllocBuffer(&b, size);
	if (!p) return NULL;
	if (ListType_Push(t, &b)) {
		ListType_Finalize(&b);
		return NULL;
	}
	return p;
}

int ListType_PushCopyBuffer(ListType *t, const void *ptr, size_t size)
{
	void *p;
	
	if (! ptr || ! (p = ListType_PushAllocBuffer(t, size))) return 1;
	memcpy(p, ptr, size);
	return 0;
}

int ListType_GetBuffer(const ListType *t, void *ptr)
{
	if (!t || !ptr || t->type != BUFFER) return 1;
	*((void **) ptr) = t->u.b;
	return 0;
}

void ListType_Finalize(ListType *t)
{
	if (t) {
		switch (t->type) {
		case LIST:
			List_Delete(t->u.l);
			break;
		case NUMBER:
			break;
		case BUFFER:
			free(t->u.b);
			break;
		}
		t->type = LIST;
		t->u.l = NULL;
	}
}

int ListType_Reverse(ListType *a)
{
	if (!a) return 1;
	if (a->type == LIST) {
		a->u.l = List_Reverse(a->u.l);
	}
	return 0;
}

List *ListType_First(ListType *c, ListType **n)
{
	if (!c || c->type != LIST) {
		if (n) *n = c;
		return NULL;
	}
	if (n) *n = &c->u.l->data;
	return c->u.l;
}

List *ListType_Next(List *c, ListType **n)
{
	if (c) {
		if (n) *n = &c->next->data;
		return c->next;
	}
	if (n) *n = NULL;
	return NULL;
}

List *List_Next(List *c)
{
	return (c) ? c->next : NULL;
}
