#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include <prsht.h>
#include <string.h>
#include <stdio.h>

#include "array.h"
#include "list.h"
#include "dialog.h"
#include "prop.h"

typedef struct {
	LPBOOL center;
	DialogTpl *lpdt;
} PageParam;

BOOL InitializeItems(HWND, DialogTpl *);
void GetResults(HWND, DialogTpl *);

static BOOL move_window_to_center(HWND hwnd)
{
	HWND hwndParent;
	RECT rcp, rc;
	int x, y;
	
	if (! hwnd) return FALSE;
	hwndParent = GetParent(hwnd);
	if (! hwndParent) {
		hwndParent = GetDesktopWindow();
		if (! hwndParent) return FALSE;
	}
	if (! GetWindowRect(hwndParent, &rcp) ||
	    ! GetWindowRect(hwnd, &rc)) {
		return FALSE;
	}
	x = rcp.left + (rcp.right - rcp.left - rc.right + rc.left) / 2;
	y = rcp.top + (rcp.bottom - rcp.top - rc.bottom + rc.top) / 2;
	if (x == rc.left && y == rc.top) return TRUE;
	return SetWindowPos(hwnd, NULL, x, y, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
}

static BOOL PropertySheet_OnInitDialog
(HWND hwnd, HWND hwndFocus, LPARAM lParam)
{
	PageParam *pp;
	DialogTpl *lpdt;
	
	pp = (PageParam *) ((PROPSHEETPAGE *) lParam)->lParam;
	if (! pp) return FALSE;
	lpdt = pp->lpdt;
	if (! lpdt) return FALSE;
	SetWindowLongPtr(hwnd, DWLP_USER, (LONG_PTR) lpdt);
	if ((LPVOID) GetWindowLongPtr(hwnd, DWLP_USER) != lpdt ||
		! InitializeItems(hwnd, lpdt)) {
		return FALSE;
	}
	if (*pp->center) {
		if (move_window_to_center(GetParent(hwnd))) {
			*pp->center = FALSE;
		}
	}
	return TRUE;
}

static void PropertySheet_OnDestroy(HWND hwnd)
{
	DialogTpl *lpdt;

	lpdt = (LPVOID) GetWindowLongPtr(hwnd, DWLP_USER);
	if (lpdt) {
		GetResults(hwnd, lpdt);
	}
}

BOOL CALLBACK
PropertySheet_DlgProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {
	case WM_INITDIALOG:
		PropertySheet_OnInitDialog(hwnd, (HWND) wParam, lParam);
		return TRUE;
	case WM_DESTROY:
		PropertySheet_OnDestroy(hwnd);
		return TRUE;
	default:
		return FALSE;
	}
}

int PropertySheet_ShowModal(HWND hwndParent, HANDLE hInstance,
							LPCTSTR pszCaption, UINT nStartPage,
							int nPages, void *lpdt)
{
	PROPSHEETHEADER psh = {0};
	PROPSHEETPAGE *psp = NULL;
	PageParam *pp = NULL;
	BOOL center = FALSE;
	int i, n, result = -1;
	
	if (! lpdt || nPages == 0) return result;
	n = (nPages < 0) ? - nPages : nPages;
	psp = (PROPSHEETPAGE *) calloc(n, sizeof(PROPSHEETPAGE));
	if (! psp) return result;
	pp = (PageParam *) calloc(n, sizeof(PageParam));
	if (! pp) {
		free(psp);
		return result;
	}
	for (i = 0; i < n; i ++) {
		psp[i].dwSize = sizeof(PROPSHEETPAGE);
		psp[i].dwFlags = PSP_DLGINDIRECT | PSP_PREMATURE;
		psp[i].hInstance = hInstance;
		psp[i].pfnDlgProc = PropertySheet_DlgProc;
		pp[i].center = &center;
		if (nPages < 0) {
			DialogTpl **ppdt = (DialogTpl **) lpdt;
			psp[i].pResource = (LPCDLGTEMPLATE) ppdt[i]->tmpl.ptr;
			pp[i].lpdt = ppdt[i];
		} else {
			DialogTpl *pdt = (DialogTpl *) lpdt;
			psp[i].pResource = (LPCDLGTEMPLATE) pdt[i].tmpl.ptr;
			pp[i].lpdt = &pdt[i];
		}
		psp[i].lParam = (LPARAM) &pp[i];
		if (psp[i].pResource->style & DS_CENTER) {
			center = TRUE;
		}
	}
	psh.dwSize = sizeof(PROPSHEETHEADER);
	psh.dwFlags = PSH_PROPSHEETPAGE | PSH_NOAPPLYNOW;
	if (pszCaption != NULL) {
		psh.dwFlags |= PSH_PROPTITLE;
	}
	psh.hwndParent = hwndParent;
	psh.hInstance = hInstance;
	psh.pszCaption = pszCaption;
	psh.nStartPage = nStartPage;
	psh.nPages = n;
	psh.ppsp = psp;
	result = PropertySheet(&psh);
	free(pp);
	free(psp);
	return result;
}

/*
  pResource member of PROPSHEETPAGE will be rewritten.
  In MSDN (http://msdn2.microsoft.com/en-us/library/bb774548.aspx)
  > PROPSHEETPAGE Structure
  > A read-only template will cause an exception in some versions of Windows.
 */
