--[[    Simple search dialog for 1.0 and kmplus

This script replaces old menu based search script
by specially designed dialog.

luamacros(search.SearchDialog, Search dialog)

]]

require "search1"
require "dialog"

search.search_history = {}
search.search_history_max = 20

local mes_caption = search.mes.prompt_caption or "Search:"
local mes_page = search.mes.prompt_newpage or "Open results in new page"
local mes_default = search.mes.prompt_setdefault or "Set default"

local prompt_dlg = create_dialog({
	x = 0,
	y = 0,
	width = 250,
	height = 65,
	caption = mes_caption,
	
	{
		type = "combobox",
		style = w32dlg.CBS_DROPDOWN + w32dlg.CBS_AUTOHSCROLL +
		        w32dlg.WS_VSCROLL + w32dlg.WS_TABSTOP,
		x = 5, y = 20, width = 235, height = 100,
		id = "ID_ANSWER",
	},
	{
		type = "button",
		style = w32dlg.BS_DEFPUSHBUTTON + w32dlg.WS_TABSTOP,
		x = 130, y = 35, width = 50, height = 14,
		id = "IDOK",
		caption = "OK",
	},
	{
		type = "button",
		style = w32dlg.BS_PUSHBUTTON + w32dlg.WS_TABSTOP,
		x = 190, y = 35, width = 50, height = 14,
		id = "IDCANCEL",
		caption = "Cancel",
	},
	{
		type = "combobox",
		style = w32dlg.WS_BORDER + w32dlg.WS_TABSTOP +
		        w32dlg.WS_VSCROLL + w32dlg.CBS_DROPDOWNLIST,
		x = 5, y = 5, width = 130, height = 100,
		id = "ID_ENGINE",
	},
	{
		type = "button",
		style = w32dlg.BS_3STATE + w32dlg.WS_TABSTOP,
		x = 140, y = 5, width = 100, height = 13,
		caption = mes_page,
		id = "ID_PAGE",
	},
	{
		type = "button",    -- control type
		style = w32dlg.BS_CHECKBOX + w32dlg.WS_TABSTOP,
		x = 5,             -- x position
		y = 35,              -- y position
		width = 100,         -- width
		height = 10,        -- height
		id = "ID_SET",    -- ID
		caption = mes_default  -- caption
	},
})

local function search_prompt(engine, how, word)
	local result, tbl
	local names, engines = {}, {}
	local esel, psel
	local k, v
	
	-- engine
	for k, v in pairs(search.engines) do
		if v.name then
			table.insert(engines, k)
			table.insert(names, v.name)
			if k == engine then
				esel = v.name
			end
		end
	end
	if not esel then
		esel = engines[1]
	end
	-- page
	if how == 1 then
		psel = w32dlg.BST_CHECKED
	elseif how == 2 then
		psel = w32dlg.BST_INDETERMINATE
	else
		psel = w32dlg.BST_UNCHECKED
	end
	-- show dialog
	result, tbl = prompt_dlg(nil, {
		ID_ANSWER = {word, search.search_history},
		ID_ENGINE = {esel, names},
		ID_PAGE = psel,
	})
	if result == "IDOK" then
		if tbl.ID_ANSWER and tbl.ID_ANSWER ~= "" then
			while #search.search_history >= search.search_history_max do
				table.remove(search.search_history)
			end
			table.insert(search.search_history, 1, tbl.ID_ANSWER)
		end
		for k, v in pairs(search.engines) do
			if tbl.ID_ENGINE == v.name then
				engine = k
				break
			end
		end
		if tbl.ID_PAGE == w32dlg.BST_UNCHECKED then
			how = 0
		elseif tbl.ID_PAGE == w32dlg.BST_CHECKED then
			how = 1
		elseif tbl.ID_PAGE == w32dlg.BST_INDETERMINATE then
			how = 2
		end
		if tbl.ID_SET == w32dlg.BST_CHECKED then
			search.SetSearch(engine)
			search.SetOpen(tostring(how))
		end
		return tbl.ID_ANSWER, engine, how
	end
end

function search.SearchDialog(engine)
	local word, how, title, url
	
	engine = search.defaultEngine(engine)
	if not engine then
		return
	end
	if engine == "URL" then
		return search.Search(engine)
	end
	-- page open
	how = getpref(TYPE_INT, search.so)
	-- selected text
	word = GetGlobalVar(TYPE_STRING, "SelectedText")
	if word and word ~= "" then
		word, engine, how = search_prompt(engine, how, word)
		if word and word ~= "" then
			search.openSearch(engine, word, how)
		end
		return
	end
	-- url bar
	word = GetGlobalVar(TYPE_STRING, "URLBAR")
	url = GetGlobalVar(TYPE_STRING, "URL")
	if word and word ~= "" and word ~= url then
		SetGlobalVar(TYPE_STRING, "URLBAR", url)
		search.openSearch(engine, word)
		return
	end
	-- prompt
	word, engine, how = search_prompt(engine, how)
	if word and word ~= "" then
		search.openSearch(engine, word, how)
	end
end
